# ------------------------------------------------------------------------------
# Compute minimum travel time multi-commodity flow
#
# Ralf Borndoerfer
# 10/2014

# Discrete Optimization in Transportation
# FU Berlin
# WS 2014

# http://www.zib.de/borndoerfer/Homepage/ws14-2.html

# ------------------------------------------------------------------------------
# input files:

# nodes.dat
# OD-nodes of the graph. Format is "1s", where
# 1s = city node
# The OD nodes are a subset of all graph nodes. We need this set for reading and
# accessing the OD-matrix.

# od.dat
# OD-matrix. Format is NxN numbers in N rows "1n 2n ... Nn", where in row i
# 1s - no of passengers that want to travel from city node i to city node 1
# 2s - no of passengers that want to travel from city node i to city node 2
# ...

# edges.dat
# Edges of the graph. Format is "1s 2s", where
# 1s = start city node
# 2s = end city node
# The edges are directed and the data contains forward and backward directions.

# cap.dat
# Edge capacities, format is "1s 2s 3n", where
# 1s = start city node
# 2s = end city node
# 3n = edge capacity

# times.dat
# Travel times, format is "1s 2s 3n", where
# 1s = start city node
# 2s = end city node
# 3n = travel time on edge

# ------------------------------------------------------------------------------
# optimal solution:
 
# SCIP Status        : problem is solved [optimal solution found]
# Solving Time (sec) : 3.10
# Solving Nodes      : 1
# Primal Bound       : +4.21373200000000e+007 (1 solutions)
# Dual Bound         : +4.21373200000000e+007
# Gap                : 0.00 %

# ------------------------------------------------------------------------------
# sets:

# OD-nodes
# The OD-nodes could be inferred from the edges "E", but
# for reading the OD-matrix we need the correct order.
set O := { read "nodes.dat" as "<1s>" comment "#" };

# edges (directed!)
set E := { read "edges.dat" as "<1s,2s>" comment "#" };

# ------------------------------------------------------------------------------
# parameters:

# OD-matrix
param d[O*O] := read "od.dat" as "n+" comment "#";

# capacities
param c[E] := read "cap.dat" as "<1s,2s> 3n" comment "#";

# times
param times[E] := read "times.dat" as "<1s,2s> 3n" comment "#";

# ------------------------------------------------------------------------------
# sets once again:

# demand pairs
set D := { <s,t> in O * O with d[s,t] > 0 };

# ------------------------------------------------------------------------------
# variables:

# y[s,t,u,v] travelers betwen <s> and <t> on edge <u,v>
var y[<s,t,u,v> in O*O*E with d[s,t] > 0] integer >= 0;

# ------------------------------------------------------------------------------
# objective:

maximize obj: sum <s,t,u,v> in O*O*E with d[s,t] > 0: times[u,v] * y[s,t,u,v];

# ------------------------------------------------------------------------------
# constraints

subto conservation:
	forall <s,t> in O*O with d[s,t] > 0:
	forall <v> in O without {t}:
	sum <u,v> in E: y[s,t,u,v] - sum <v,w> in E: y[s,t,v,w] == 
	if v == s then d[s,t] else 0 end;

subto capacity:
	forall <u,v> in E:
	sum <s,t> in O*O with d[s,t] > 0: y[s,t,u,v] <= c[u,v];
